import numpy as np

from aviary.subsystems.propulsion.engine_deck import EngineDeck
from aviary.utils.aviary_values import AviaryValues
from aviary.utils.preprocessors import preprocess_propulsion
from aviary.utils.functions import get_path
from aviary.variable_info.enums import EquationsOfMotion, LegacyCode
from aviary.variable_info.variables import Aircraft, Mission, Settings

LargeSingleAisle1FLOPS = {}
inputs = LargeSingleAisle1FLOPS['inputs'] = AviaryValues()
outputs = LargeSingleAisle1FLOPS['outputs'] = AviaryValues()

# Overall Aircraft
# ---------------------------
inputs.set_val(Aircraft.Design.BASE_AREA, 0., 'ft**2')
inputs.set_val(Aircraft.Design.EMPTY_MASS_MARGIN_SCALER, 0.0)
inputs.set_val(Aircraft.Design.TOUCHDOWN_MASS, 152800.0, 'lbm')
inputs.set_val(Mission.Design.GROSS_MASS, 181200., 'lbm')
inputs.set_val(Aircraft.Design.USE_ALT_MASS, False)
inputs.set_val(Aircraft.Design.LIFT_DEPENDENT_DRAG_COEFF_FACTOR, 0.909839381134961)
inputs.set_val(Aircraft.Design.SUBSONIC_DRAG_COEFF_FACTOR, 1.0)
inputs.set_val(Aircraft.Design.SUPERSONIC_DRAG_COEFF_FACTOR, 1.0)
inputs.set_val(Aircraft.Design.ZERO_LIFT_DRAG_COEFF_FACTOR, 0.930890028006548)

# Air Conditioning
# ---------------------------
inputs.set_val(Aircraft.AirConditioning.MASS_SCALER, 1.0)

# Anti-Icing
# ---------------------------
inputs.set_val(Aircraft.AntiIcing.MASS_SCALER, 1.0)

# APU
# ---------------------------
inputs.set_val(Aircraft.APU.MASS_SCALER, 1.1)

# Avionics
# ---------------------------
inputs.set_val(Aircraft.Avionics.MASS_SCALER, 1.2)

# Canard
# ---------------------------
inputs.set_val(Aircraft.Canard.AREA, 0.0, 'ft**2')
inputs.set_val(Aircraft.Canard.ASPECT_RATIO, 0.0)
inputs.set_val(Aircraft.Canard.THICKNESS_TO_CHORD, 0.0)

# Crew and Payload
# ---------------------------
inputs.set_val(Aircraft.CrewPayload.NUM_BUSINESS_CLASS, 0)
inputs.set_val(Aircraft.CrewPayload.CARGO_CONTAINER_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.CrewPayload.NUM_FIRST_CLASS, 11)
inputs.set_val(Aircraft.CrewPayload.NUM_FLIGHT_ATTENDANTS, 3)
inputs.set_val(Aircraft.CrewPayload.NUM_FLIGHT_CREW, 2)
inputs.set_val(Aircraft.CrewPayload.FLIGHT_CREW_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.CrewPayload.NUM_GALLEY_CREW, 0)
inputs.set_val(Aircraft.CrewPayload.MISC_CARGO, 0., 'lbm')
inputs.set_val(Aircraft.CrewPayload.NON_FLIGHT_CREW_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.CrewPayload.NUM_PASSENGERS, 169, units='unitless')
inputs.set_val(Aircraft.CrewPayload.PASSENGER_SERVICE_MASS_SCALER, 1.)
inputs.set_val(Aircraft.CrewPayload.NUM_TOURIST_CLASS, 158)
inputs.set_val(Aircraft.CrewPayload.MASS_PER_PASSENGER, 180., 'lbm')
inputs.set_val(Aircraft.CrewPayload.WING_CARGO, 0., 'lbm')

# Electrical
# ---------------------------
inputs.set_val(Aircraft.Electrical.MASS_SCALER, 1.25)

# Fins
# ---------------------------
inputs.set_val(Aircraft.Fins.AREA, 0.0, 'ft**2')
inputs.set_val(Aircraft.Fins.NUM_FINS, 0)
inputs.set_val(Aircraft.Fins.TAPER_RATIO, 10.0)
inputs.set_val(Aircraft.Fins.MASS, 0.0, 'lbm')
inputs.set_val(Aircraft.Fins.MASS_SCALER, 1.0)

# Fuel
# ---------------------------
inputs.set_val(Aircraft.Fuel.AUXILIARY_FUEL_CAPACITY, 0.0, 'lbm')
inputs.set_val(Aircraft.Fuel.DENSITY_RATIO, 1.0)
inputs.set_val(Aircraft.Fuel.FUEL_SYSTEM_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Fuel.FUSELAGE_FUEL_CAPACITY, 0.0, 'lbm')
inputs.set_val(Aircraft.Fuel.NUM_TANKS, 7)
inputs.set_val(Aircraft.Fuel.TOTAL_CAPACITY, 45694.0, 'lbm')
inputs.set_val(Aircraft.Fuel.UNUSABLE_FUEL_MASS_SCALER, 1.0)

# Furnishings
# ---------------------------
inputs.set_val(Aircraft.Furnishings.MASS_SCALER, 1.1)

# Fuselage
# ---------------------------
inputs.set_val(Aircraft.Fuselage.NUM_FUSELAGES, 1)
inputs.set_val(Aircraft.Fuselage.LENGTH, 128., 'ft')
inputs.set_val(Aircraft.Fuselage.PASSENGER_COMPARTMENT_LENGTH, 85.5, 'ft')
inputs.set_val(Aircraft.Fuselage.MAX_HEIGHT, 13.17, 'ft')
inputs.set_val(Aircraft.Fuselage.MAX_WIDTH, 12.33, 'ft')
inputs.set_val(Aircraft.Fuselage.MILITARY_CARGO_FLOOR, False)
inputs.set_val(Aircraft.Fuselage.PLANFORM_AREA, 128.0 * 12.33, 'ft**2')
inputs.set_val(Aircraft.Fuselage.MASS_SCALER, 1.05)
inputs.set_val(Aircraft.Fuselage.WETTED_AREA, 4158.62, 'ft**2')  # Override
inputs.set_val(Aircraft.Fuselage.WETTED_AREA_SCALER, 1.0)

# Horizontal Tail
# ---------------------------
inputs.set_val(Aircraft.HorizontalTail.AREA, 355., 'ft**2')
inputs.set_val(Aircraft.HorizontalTail.ASPECT_RATIO, 6.0)
inputs.set_val(Aircraft.HorizontalTail.TAPER_RATIO, 0.22)
inputs.set_val(Aircraft.HorizontalTail.THICKNESS_TO_CHORD, 0.125)
inputs.set_val(Aircraft.HorizontalTail.VERTICAL_TAIL_FRACTION, 0.0)
inputs.set_val(Aircraft.HorizontalTail.MASS_SCALER, 1.2)
inputs.set_val(Aircraft.HorizontalTail.WETTED_AREA, 592.65, 'ft**2')  # Override
inputs.set_val(Aircraft.HorizontalTail.WETTED_AREA_SCALER, 1.0)

# Hydraulics
# ---------------------------
inputs.set_val(Aircraft.Hydraulics.SYSTEM_PRESSURE, 3000., 'lbf/ft**2')
inputs.set_val(Aircraft.Hydraulics.MASS_SCALER, 1.0)

# Instruments
# ---------------------------
inputs.set_val(Aircraft.Instruments.MASS_SCALER, 1.25)

# Landing Gear
# ---------------------------
inputs.set_val(Aircraft.LandingGear.CARRIER_BASED, False)
inputs.set_val(Aircraft.LandingGear.MAIN_GEAR_OLEO_LENGTH, 102.0, 'inch')
inputs.set_val(Aircraft.LandingGear.MAIN_GEAR_MASS_SCALER, 1.1)
inputs.set_val(Aircraft.LandingGear.NOSE_GEAR_OLEO_LENGTH, 67.0, 'inch')
inputs.set_val(Aircraft.LandingGear.NOSE_GEAR_MASS_SCALER, 1.0)

# Nacelle
# ---------------------------
inputs.set_val(Aircraft.Nacelle.AVG_DIAMETER, 7.94, 'ft')
inputs.set_val(Aircraft.Nacelle.AVG_LENGTH, 12.3, 'ft')
inputs.set_val(Aircraft.Nacelle.MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Nacelle.WETTED_AREA_SCALER, 1.0)

# Paint
# ---------------------------
inputs.set_val(Aircraft.Paint.MASS_PER_UNIT_AREA, 0.037, 'lbm/ft**2')

# Propulsion and Engine
# ---------------------------
inputs.set_val(Aircraft.Propulsion.ENGINE_OIL_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Propulsion.MISC_MASS_SCALER, 1.0)

filename = get_path(
    'models/engines/turbofan_28k.deck')

engine_inputs = AviaryValues()
engine_inputs.set_val(Aircraft.Engine.DATA_FILE, filename)
engine_mass = 7400
engine_mass_units = 'lbm'
engine_inputs.set_val(Aircraft.Engine.MASS, engine_mass, engine_mass_units)
engine_inputs.set_val(
    Aircraft.Engine.REFERENCE_MASS,
    engine_mass,
    engine_mass_units)
scaled_sls_thrust = 28928.1
scaled_sls_thrust_units = 'lbf'
engine_inputs.set_val(
    Aircraft.Engine.SCALED_SLS_THRUST, scaled_sls_thrust, scaled_sls_thrust_units
)
engine_inputs.set_val(
    Aircraft.Engine.REFERENCE_SLS_THRUST, scaled_sls_thrust, scaled_sls_thrust_units)
num_engines = 2
engine_inputs.set_val(Aircraft.Engine.NUM_ENGINES, num_engines)
num_fuselage_engines = 0
engine_inputs.set_val(Aircraft.Engine.NUM_FUSELAGE_ENGINES, 0)
num_wing_engines = num_engines
engine_inputs.set_val(Aircraft.Engine.NUM_WING_ENGINES, num_wing_engines)
engine_inputs.set_val(Aircraft.Engine.THRUST_REVERSERS_MASS_SCALER, 0.0)
engine_inputs.set_val(
    Aircraft.Engine.WING_LOCATIONS, 15.8300 / (117.83 / 2))
engine_inputs.set_val(Aircraft.Engine.SCALE_MASS, True)
engine_inputs.set_val(Aircraft.Engine.MASS_SCALER, 1.15)
engine_inputs.set_val(Aircraft.Engine.SCALE_PERFORMANCE, True)
engine_inputs.set_val(Aircraft.Engine.SUBSONIC_FUEL_FLOW_SCALER, 1.0)
engine_inputs.set_val(Aircraft.Engine.SUPERSONIC_FUEL_FLOW_SCALER, 1.0)
engine_inputs.set_val(
    Aircraft.Engine.FUEL_FLOW_SCALER_CONSTANT_TERM, 0.0)
engine_inputs.set_val(Aircraft.Engine.FUEL_FLOW_SCALER_LINEAR_TERM, 0.0)
engine_inputs.set_val(Aircraft.Engine.CONSTANT_FUEL_CONSUMPTION, 0.0, units='lbm/h')
engine_inputs.set_val(Aircraft.Engine.ADDITIONAL_MASS_FRACTION, 0.0)
engine_inputs.set_val(Aircraft.Engine.GENERATE_FLIGHT_IDLE, True)
engine_inputs.set_val(Aircraft.Engine.IGNORE_NEGATIVE_THRUST, False)
engine_inputs.set_val(Aircraft.Engine.FLIGHT_IDLE_THRUST_FRACTION, 0.0)
engine_inputs.set_val(Aircraft.Engine.FLIGHT_IDLE_MAX_FRACTION, 1.0)
engine_inputs.set_val(Aircraft.Engine.FLIGHT_IDLE_MIN_FRACTION, 0.08)
engine_inputs.set_val(Aircraft.Engine.GEOPOTENTIAL_ALT, False)
engine_inputs.set_val(Aircraft.Engine.INTERPOLATION_METHOD, 'slinear')


# Vertical Tail
# ---------------------------
inputs.set_val(Aircraft.VerticalTail.NUM_TAILS, 1)
inputs.set_val(Aircraft.VerticalTail.AREA, 284., 'ft**2')
inputs.set_val(Aircraft.VerticalTail.ASPECT_RATIO, 1.75)
inputs.set_val(Aircraft.VerticalTail.TAPER_RATIO, 0.33)
inputs.set_val(Aircraft.VerticalTail.THICKNESS_TO_CHORD, 0.1195)
inputs.set_val(Aircraft.VerticalTail.MASS_SCALER, 1.0)
inputs.set_val(Aircraft.VerticalTail.WETTED_AREA, 581.13, 'ft**2')  # Override
inputs.set_val(Aircraft.VerticalTail.WETTED_AREA_SCALER, 1.0)

# Wing
# ---------------------------
inputs.set_val(Aircraft.Wing.AEROELASTIC_TAILORING_FACTOR, 0.0)
inputs.set_val(Aircraft.Wing.AIRFOIL_TECHNOLOGY, 1.92669766647637)
inputs.set_val(Aircraft.Wing.AREA, 1370.0, 'ft**2')
inputs.set_val(Aircraft.Wing.ASPECT_RATIO, 11.22091)
inputs.set_val(Aircraft.Wing.BENDING_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Wing.CHORD_PER_SEMISPAN_DIST, np.array([0.31, 0.23, 0.084]))
inputs.set_val(Aircraft.Wing.COMPOSITE_FRACTION, 0.2)
inputs.set_val(Aircraft.Wing.CONTROL_SURFACE_AREA, 137, 'ft**2')
inputs.set_val(Aircraft.Wing.CONTROL_SURFACE_AREA_RATIO, 0.1)
inputs.set_val(Aircraft.Wing.GLOVE_AND_BAT, 134.0, 'ft**2')
inputs.set_val(Aircraft.Wing.INPUT_STATION_DIST, np.array([0.0, 0.2759, 0.9367]))
inputs.set_val(Aircraft.Wing.LOAD_DISTRIBUTION_CONTROL, 2.0)
inputs.set_val(Aircraft.Wing.LOAD_FRACTION, 1.0)
inputs.set_val(Aircraft.Wing.LOAD_PATH_SWEEP_DIST, np.array([0.0, 22.0]), 'deg')
inputs.set_val(Aircraft.Wing.MAX_CAMBER_AT_70_SEMISPAN, 0.0)
inputs.set_val(Aircraft.Wing.MISC_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Wing.NUM_INTEGRATION_STATIONS, 50)
inputs.set_val(Aircraft.Wing.SHEAR_CONTROL_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Wing.CONTROL_SURFACE_AREA_RATIO, 0.1)
inputs.set_val(Aircraft.Wing.SPAN, 117.83, 'ft')
inputs.set_val(Aircraft.Wing.SPAN_EFFICIENCY_REDUCTION, False)
inputs.set_val(Aircraft.Wing.STRUT_BRACING_FACTOR, 0.0)
inputs.set_val(Aircraft.Wing.SURFACE_CONTROL_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Wing.SWEEP, 25.0, 'deg')
inputs.set_val(Aircraft.Wing.TAPER_RATIO, 0.27800)
inputs.set_val(Aircraft.Wing.THICKNESS_TO_CHORD, 0.1300)
inputs.set_val(
    Aircraft.Wing.THICKNESS_TO_CHORD_DIST, np.array([0.145, 0.115, 0.104]))
inputs.set_val(Aircraft.Wing.ULTIMATE_LOAD_FACTOR, 3.75)
inputs.set_val(Aircraft.Wing.VAR_SWEEP_MASS_PENALTY, 0.0)
inputs.set_val(Aircraft.Wing.MASS_SCALER, 1.23)
inputs.set_val(Aircraft.Wing.WETTED_AREA, 2396.56, 'ft**2')  # Override
inputs.set_val(Aircraft.Wing.WETTED_AREA_SCALER, 1.0)

# Mission
# ---------------------------
inputs.set_val(Mission.Summary.CRUISE_MACH, 0.785)
inputs.set_val(Mission.Summary.FUEL_FLOW_SCALER, 1.0)
inputs.set_val(Mission.Design.RANGE, 3500, 'NM')
inputs.set_val(Mission.Constraints.MAX_MACH, 0.785)
# TODO investigate the origin of these values (taken from benchmark tests)
# TODO: where should this get connected from?
inputs.set_val(Mission.Takeoff.FUEL_SIMPLE, 577, 'lbm')

# region TODO: should this come from aero?
inputs.set_val(Mission.Landing.LIFT_COEFFICIENT_MAX, 3)
inputs.set_val(Mission.Takeoff.LIFT_COEFFICIENT_MAX, 2)
inputs.set_val(Mission.Takeoff.LIFT_OVER_DRAG, 17.354)
# endregion TODO: should this come from aero?

# TODO: should this be a user input or should it be hard coded somewhere assuming it will
# # never change?
inputs.set_val(Mission.Takeoff.ROLLING_FRICTION_COEFFICIENT, .0175)
# lbf TODO: where should this get connected from?
inputs.set_val(Mission.Design.THRUST_TAKEOFF_PER_ENG, 28928.0, 'lbf')

# Settings
# ---------------------------
inputs.set_val(Settings.EQUATIONS_OF_MOTION, EquationsOfMotion.HEIGHT_ENERGY)
inputs.set_val(Settings.MASS_METHOD, LegacyCode.FLOPS)

# ---------------------------
#          OUTPUTS
# ---------------------------

outputs.set_val(Aircraft.Design.EMPTY_MASS, 92023., 'lbm')
outputs.set_val(Aircraft.Design.EMPTY_MASS_MARGIN, 0.0, 'lbm')
outputs.set_val(Aircraft.Design.OPERATING_MASS, 97992., 'lbm')
outputs.set_val(Aircraft.Propulsion.MASS, 16118., 'lbm')
outputs.set_val(Aircraft.Design.STRUCTURE_MASS, 50736., 'lbm')
outputs.set_val(Aircraft.Design.SYSTEMS_EQUIP_MASS, 25169., 'lbm')
outputs.set_val(Aircraft.Design.TOTAL_WETTED_AREA, 8275.86, 'ft**2')
outputs.set_val(Aircraft.Design.ZERO_FUEL_MASS, 135848., 'lbm')
outputs.set_val(Mission.Design.FUEL_MASS, 45352., 'lbm')

outputs.set_val(Aircraft.AirConditioning.MASS, 1602., 'lbm')

outputs.set_val(Aircraft.AntiIcing.MASS, 208.85, 'lbm')

outputs.set_val(Aircraft.APU.MASS, 1142., 'lbm')

outputs.set_val(Aircraft.Avionics.MASS, 1652.6, 'lbm')

outputs.set_val(Aircraft.Canard.CHARACTERISTIC_LENGTH, 0.0, 'ft')
outputs.set_val(Aircraft.Canard.FINENESS, 0.0)
outputs.set_val(Aircraft.Canard.WETTED_AREA, 0.0, 'ft**2')
outputs.set_val(Aircraft.Canard.MASS, 0.0, 'lbm')

outputs.set_val(Aircraft.CrewPayload.BAGGAGE_MASS, 7436., 'lbm')
outputs.set_val(Aircraft.CrewPayload.CARGO_MASS, 0., 'lbm')
outputs.set_val(Aircraft.CrewPayload.CARGO_CONTAINER_MASS, 1400., 'lbm')
outputs.set_val(Aircraft.CrewPayload.FLIGHT_CREW_MASS, 450.0, 'lbm')
outputs.set_val(Aircraft.CrewPayload.NON_FLIGHT_CREW_MASS, 465.0, 'lbm')
outputs.set_val(Aircraft.CrewPayload.PASSENGER_SERVICE_MASS, 3022.74805809, 'lbm')
outputs.set_val(Aircraft.CrewPayload.PASSENGER_MASS, 30420., 'lbm')
outputs.set_val(Aircraft.CrewPayload.TOTAL_PAYLOAD_MASS, 37856., 'lbm')

outputs.set_val(Aircraft.Electrical.MASS, 2464.0, 'lbm')

outputs.set_val(Aircraft.Fuel.FUEL_SYSTEM_MASS, 669.58, 'lbm')
outputs.set_val(Aircraft.Fuel.UNUSABLE_FUEL_MASS, 501.3, 'lbm')

outputs.set_val(Aircraft.Fins.MASS, 0.0, 'lbm')

outputs.set_val(Aircraft.Furnishings.MASS, 15517., 'lbm')

avg_diameter = 12.75
avg_diameter_units = 'ft'
outputs.set_val(Aircraft.Fuselage.AVG_DIAMETER, avg_diameter, avg_diameter_units)
outputs.set_val(Aircraft.Fuselage.CHARACTERISTIC_LENGTH, 128., 'ft')
outputs.set_val(
    Aircraft.Fuselage.CROSS_SECTION,
    np.pi * (avg_diameter / 2.0)**2.0, f'{avg_diameter_units}**2')
outputs.set_val(Aircraft.Fuselage.DIAMETER_TO_WING_SPAN, 0.108207)
outputs.set_val(Aircraft.Fuselage.FINENESS, 10.0392)
outputs.set_val(Aircraft.Fuselage.LENGTH_TO_DIAMETER, 10.039216)
outputs.set_val(Aircraft.Fuselage.MASS, 18357., 'lbm')

outputs.set_val(Aircraft.HorizontalTail.CHARACTERISTIC_LENGTH, 7.69, 'ft')
outputs.set_val(Aircraft.HorizontalTail.FINENESS, 0.1250)
outputs.set_val(Aircraft.HorizontalTail.MASS, 1831., 'lbm')

outputs.set_val(Aircraft.Hydraulics.MASS, 1086.7, 'lbm')

outputs.set_val(Aircraft.Instruments.MASS, 601, 'lbm')

outputs.set_val(Aircraft.LandingGear.MAIN_GEAR_MASS, 7910.32, 'lbm')
outputs.set_val(Aircraft.LandingGear.NOSE_GEAR_MASS, 870.59, 'lbm')

outputs.set_val(Aircraft.Nacelle.CHARACTERISTIC_LENGTH, np.array([12.30]), 'ft')
outputs.set_val(Aircraft.Nacelle.FINENESS, np.array([1.5491]))
outputs.set_val(Aircraft.Nacelle.MASS, 1971.4, 'lbm')

nacelle_wetted_area = np.array([273.45])
nacelle_wetted_area_units = 'ft**2'
outputs.set_val(
    Aircraft.Nacelle.WETTED_AREA, nacelle_wetted_area, nacelle_wetted_area_units)

outputs.set_val(
    Aircraft.Nacelle.TOTAL_WETTED_AREA,
    2 * nacelle_wetted_area, nacelle_wetted_area_units)

outputs.set_val(Aircraft.Paint.MASS, 306.2, 'lbm')

outputs.set_val(
    Aircraft.Propulsion.TOTAL_SCALED_SLS_THRUST,
    scaled_sls_thrust * num_engines, scaled_sls_thrust_units)

outputs.set_val(
    Aircraft.Propulsion.TOTAL_NUM_ENGINES, num_engines)

engine_ctrls_mass = 88.44
engine_ctrls_mass_units = 'lbm'
outputs.set_val(
    Aircraft.Engine.CONTROLS_MASS, engine_ctrls_mass, engine_ctrls_mass_units)
outputs.set_val(
    Aircraft.Propulsion.TOTAL_ENGINE_CONTROLS_MASS,
    engine_ctrls_mass, engine_ctrls_mass_units)

outputs.set_val(Aircraft.Propulsion.TOTAL_ENGINE_OIL_MASS, 130.23, 'lbm')
outputs.set_val(Aircraft.Propulsion.TOTAL_NUM_WING_ENGINES, num_wing_engines)

outputs.set_val(Aircraft.Propulsion.TOTAL_NUM_FUSELAGE_ENGINES, num_fuselage_engines)

outputs.set_val(Aircraft.Engine.MASS, 14800/2, 'lbm')
outputs.set_val(Aircraft.Engine.POD_MASS, 9000, 'lbm')
outputs.set_val(Aircraft.Engine.ADDITIONAL_MASS, 0.0, 'lbm')
outputs.set_val(Aircraft.Propulsion.TOTAL_MISC_MASS, 648.83, 'lbm')
outputs.set_val(Aircraft.Engine.SCALE_FACTOR, 1.0)
outputs.set_val(Aircraft.Propulsion.TOTAL_STARTER_MASS, 560.39, 'lbm')

outputs.set_val(Aircraft.Propulsion.TOTAL_THRUST_REVERSERS_MASS, 0, 'lbm')
outputs.set_val(Aircraft.Engine.THRUST_REVERSERS_MASS, 0, 'lbm')

outputs.set_val(
    Aircraft.Propulsion.TOTAL_ENGINE_MASS,
    engine_mass * num_engines, engine_mass_units)

outputs.set_val(Aircraft.VerticalTail.CHARACTERISTIC_LENGTH, 12.74, 'ft')
outputs.set_val(Aircraft.VerticalTail.FINENESS, 0.1195)
outputs.set_val(Aircraft.VerticalTail.MASS, 1221.8, 'lbm')

outputs.set_val(Aircraft.Wing.BENDING_FACTOR, 11.5918)
outputs.set_val(Aircraft.Wing.BENDING_MASS, 8184.8, 'lbm')
outputs.set_val(Aircraft.Wing.CHARACTERISTIC_LENGTH, 10.49, 'ft')
# Not in FLOPS output; calculated from inputs.
outputs.set_val(Aircraft.Wing.CONTROL_SURFACE_AREA, 137, 'ft**2')
outputs.set_val(Aircraft.Wing.ENG_POD_INERTIA_FACTOR, 0.967333)
outputs.set_val(Aircraft.Wing.FINENESS, 0.1300)
outputs.set_val(Aircraft.Wing.MISC_MASS, 1668.3, 'lbm')
outputs.set_val(Aircraft.Wing.SHEAR_CONTROL_MASS, 4998.8, 'lbm')
outputs.set_val(Aircraft.Wing.SURFACE_CONTROL_MASS, 894, 'lbm')

outputs.set_val(Aircraft.Wing.MASS, 18268, 'lbm')

outputs.set_val(Mission.Design.MACH, 0.800)
outputs.set_val(Mission.Design.LIFT_COEFFICIENT, 0.568)

# Create engine model
engine = EngineDeck(name='engine',
                    options=engine_inputs
                    )
preprocess_propulsion(inputs, [engine])



import openmdao.api as om
from openmdao.utils.assert_utils import assert_near_equal


def check_prob_outputs(prob, vals, rtol=1e-6):
    """Check multiple problem outputs and print all failures.

    testflo doesn't handle unittest subTests, so this is a way to get all the
    failures at once without needing to iteratively fix and re-run. It also prints the
    variable path so you can more easily tell which assertions failed.

    Parameters
    ----------
    prob : Problem
        Problem instance to check. Should have already been run.
    vals : dict
        Dictionary mapping variable paths (i.e. ``prob[path]`` should work) to the
        value(s) to check against.
    rtol : float or list, optional
        Maximum relative tolerance to pass the test. If given as a list, it should match
        the number of entries in `vals`.
    """
    if isinstance(rtol, float):
        rtol = len(vals) * [rtol]

    errors = []
    for (path, val), t in zip(vals.items(), rtol):
        try:
            assert_near_equal(prob[path], val, tolerance=t)
        except ValueError as e:
            errors.append(f"\n  {path}: {e}")

    if errors:
        raise ValueError("".join(errors))



import unittest

import openmdao.api as om
from parameterized import parameterized

from aviary.subsystems.mass.flops_based.furnishings import (
    AltFurnishingsGroupMass, AltFurnishingsGroupMassBase,
    BWBFurnishingsGroupMass, TransportFurnishingsGroupMass)
from aviary.utils.test_utils.variable_test import assert_match_varnames
from aviary.validation_cases.validation_tests import (Version,
                                                      flops_validation_test,
                                                      get_flops_case_names,
                                                      get_flops_inputs,
                                                      print_case)
from aviary.variable_info.variables import Aircraft, Mission


class TransportFurnishingsGroupMassTest(unittest.TestCase):
    '''
    Tests transport/GA furnishings mass calculation.
    '''

    def setUp(self):
        self.prob = om.Problem()

    @parameterized.expand(get_flops_case_names(),
                          name_func=print_case)
    def test_case(self, case_name):

        prob = self.prob

        prob.model.add_subsystem(
            'furnishings',
            TransportFurnishingsGroupMass(
                aviary_options=get_flops_inputs(case_name, preprocess=True)),
            promotes_outputs=['*'],
            promotes_inputs=['*']
        )

        prob.setup(check=False, force_alloc_complex=True)

        flops_validation_test(
            prob,
            case_name,
            input_keys=[Aircraft.Furnishings.MASS_SCALER,
                        Aircraft.Fuselage.PASSENGER_COMPARTMENT_LENGTH,
                        Aircraft.Fuselage.MAX_WIDTH,
                        Aircraft.Fuselage.MAX_HEIGHT],
            output_keys=Aircraft.Furnishings.MASS,
            version=Version.TRANSPORT)

    def test_IO(self):
        assert_match_varnames(self.prob.model)


class BWBFurnishingsGroupMassTest(unittest.TestCase):
    '''
    Tests BWB furnishings mass calculation.
    '''

    def setUp(self):
        self.prob = om.Problem()

    @parameterized.expand(get_flops_case_names(),
                          name_func=print_case)
    def test_case(self, case_name):

        prob = self.prob
        flops_inputs = get_flops_inputs(case_name, preprocess=True)
        flops_inputs.update({
            Aircraft.Fuselage.MILITARY_CARGO_FLOOR: (False, 'unitless'),
            Aircraft.BWB.NUM_BAYS: (5, 'unitless')
        })

        prob.model.add_subsystem(
            'furnishings',
            BWBFurnishingsGroupMass(aviary_options=flops_inputs),
            promotes_outputs=['*'],
            promotes_inputs=['*']
        )

        prob.setup(check=False, force_alloc_complex=True)

        # TODO: add FLOPS tests cases with BWB furnishings mass calculations

        # These inputs aren't in the FLOPS input data so we'll give dummy values here,
        # instead of trying to transfer them from the FLOPS input data. The test
        # case will only check the partials.
        prob.set_val(Aircraft.BWB.CABIN_AREA, 1000.0, units='ft**2')
        prob.set_val(Aircraft.BWB.PASSENGER_LEADING_EDGE_SWEEP, 30.0, units='deg')

        flops_validation_test(
            prob,
            case_name,
            input_keys=[Aircraft.Furnishings.MASS_SCALER,
                        # Aircraft.BWB.CABIN_AREA,
                        # Aircraft.BWB.PASSENGER_LEADING_EDGE_SWEEP,
                        Aircraft.Fuselage.MAX_WIDTH,
                        Aircraft.Fuselage.MAX_HEIGHT],
            output_keys=Aircraft.AirConditioning.MASS,
            version=Version.BWB,
            tol=1.0e-3,
            atol=1e-11)

    def test_IO(self):
        assert_match_varnames(self.prob.model)


class AltFurnishingsGroupMassBaseTest(unittest.TestCase):
    '''
    Tests alternate base furnishings mass calculation.
    '''

    def setUp(self):
        self.prob = om.Problem()

    @parameterized.expand(get_flops_case_names(),
                          name_func=print_case)
    def test_case(self, case_name):

        prob = self.prob

        prob.model.add_subsystem(
            'furnishings',
            AltFurnishingsGroupMassBase(
                aviary_options=get_flops_inputs(case_name, preprocess=True)),
            promotes_outputs=['*'],
            promotes_inputs=['*']
        )

        prob.setup(check=False, force_alloc_complex=True)

        flops_validation_test(
            prob,
            case_name,
            input_keys=Aircraft.Furnishings.MASS_SCALER,
            output_keys=Aircraft.Furnishings.MASS_BASE,
            version=Version.ALTERNATE)

    def test_IO(self):
        assert_match_varnames(self.prob.model)


class AltFurnishingsGroupMassTest(unittest.TestCase):
    '''
    Tests alternate furnishings mass calculation.
    '''

    def setUp(self):
        self.prob = om.Problem()

    @parameterized.expand(get_flops_case_names(),
                          name_func=print_case)
    def test_case(self, case_name):

        prob = self.prob

        prob.model.add_subsystem(
            'furnishings',
            AltFurnishingsGroupMass(
                aviary_options=get_flops_inputs(case_name, preprocess=True)),
            promotes_outputs=['*'],
            promotes_inputs=['*']
        )

        prob.setup(check=False, force_alloc_complex=True)

        flops_validation_test(
            prob,
            case_name,
            input_keys=[Aircraft.Furnishings.MASS_BASE,
                        Aircraft.Design.STRUCTURE_MASS,
                        Aircraft.Propulsion.MASS,
                        Aircraft.Design.SYSTEMS_EQUIP_MASS_BASE],
            output_keys=Aircraft.Furnishings.MASS,
            version=Version.ALTERNATE)

    def test_IO(self):
        assert_match_varnames(self.prob.model)


if __name__ == '__main__':
    unittest.main()



import unittest
import os

import numpy as np
import openmdao
import openmdao.api as om
from openmdao.utils.assert_utils import assert_check_partials
from packaging import version

from aviary.mission.gasp_based.ode.descent_ode import DescentODE
from aviary.variable_info.options import get_option_defaults
from aviary.utils.test_utils.IO_test_util import check_prob_outputs
from aviary.variable_info.enums import SpeedType
from aviary.variable_info.variables import Dynamic
from aviary.interface.default_phase_info.two_dof import default_mission_subsystems


class DescentODETestCase(unittest.TestCase):
    def setUp(self):
        self.prob = om.Problem()
        self.sys = self.prob.model = DescentODE(num_nodes=1,
                                                mach_cruise=0.8,
                                                aviary_options=get_option_defaults(),
                                                core_subsystems=default_mission_subsystems)

    @unittest.skipIf(version.parse(openmdao.__version__) < version.parse("3.26"), "Skipping due to OpenMDAO version being too low (<3.26)")
    def test_high_alt(self):
        """Test descent above 10k ft with Mach under and over the EAS limit"""
        self.sys.options["num_nodes"] = 2
        self.sys.options["input_speed_type"] = SpeedType.MACH
        self.sys.options["EAS_limit"] = 350

        self.prob.setup(check=False, force_alloc_complex=True)

        self.prob.set_val(
            Dynamic.Mission.THROTTLE, np.array([
                0, 0]), units='unitless')
        self.prob.set_val(Dynamic.Mission.ALTITUDE, np.array([36500, 14500]), units="ft")
        self.prob.set_val(Dynamic.Mission.MASS, np.array([147661, 147572]), units="lbm")

        self.prob.run_model()

        testvals = {
            "alpha": np.array([3.2, 1.21]),
            "CL": np.array([0.5123, 0.2583]),
            "CD": np.array([0.0279, 0.0197]),
            Dynamic.Mission.ALTITUDE_RATE: np.array([-2385, -3076]) / 60,
            # TAS (ft/s) * cos(gamma)
            Dynamic.Mission.DISTANCE_RATE: [
                (459 * 1.68781) * np.cos(np.deg2rad(-2.94)),
                (437 * 1.68781) * np.cos(np.deg2rad(-3.98)),
            ],
            # lbm/h
            Dynamic.Mission.FUEL_FLOW_RATE_NEGATIVE_TOTAL: np.array([-452, -996]),
            "EAS": np.array([249, 350]) * 1.68781,  # kts -> ft/s
            Dynamic.Mission.MACH: np.array([0.8, 0.696]),
            Dynamic.Mission.FLIGHT_PATH_ANGLE: np.deg2rad([-2.94, -3.98]),
        }
        check_prob_outputs(self.prob, testvals, rtol=1e-1)  # TODO tighten

        partial_data = self.prob.check_partials(
            method="cs", out_stream=None, excludes=["*USatm*", "*params*", "*aero*"]
        )
        assert_check_partials(partial_data, atol=1e-8, rtol=1e-8)

    def test_low_alt(self):
        """Test descent below 10k ft"""
        self.sys.options["input_speed_type"] = SpeedType.EAS
        self.sys.options["EAS_limit"] = 350

        self.prob.setup(check=False, force_alloc_complex=True)

        self.prob.set_val(Dynamic.Mission.THROTTLE, 0, units='unitless')
        self.prob.set_val(Dynamic.Mission.ALTITUDE, 1500, units="ft")
        self.prob.set_val(Dynamic.Mission.MASS, 147410, units="lbm")
        self.prob.set_val("EAS", 250, units="kn")

        self.prob.run_model()

        testvals = {
            "alpha": 4.21,
            "CL": 0.5063,
            "CD": 0.0271,
            Dynamic.Mission.ALTITUDE_RATE: -1158 / 60,
            # TAS (ft/s) * cos(gamma)
            Dynamic.Mission.DISTANCE_RATE: (255 * 1.68781) * np.cos(np.deg2rad(-2.56)),
            Dynamic.Mission.FUEL_FLOW_RATE_NEGATIVE_TOTAL: -1294,
            Dynamic.Mission.FLIGHT_PATH_ANGLE: np.deg2rad(-2.56),
        }
        check_prob_outputs(self.prob, testvals, rtol=1e-1)  # TODO tighten

        partial_data = self.prob.check_partials(
            out_stream=None, method="cs", excludes=["*USatm*", "*params*", "*aero*"]
        )
        assert_check_partials(partial_data, atol=1e-8, rtol=1e-8)


if __name__ == "__main__":
    unittest.main()



import unittest

import numpy as np
import openmdao.api as om
from openmdao.utils.assert_utils import assert_near_equal
from parameterized import parameterized

from aviary.subsystems.mass.flops_based.engine import EngineMass
from aviary.subsystems.propulsion.engine_deck import EngineDeck
from aviary.utils.aviary_values import AviaryValues
from aviary.utils.preprocessors import preprocess_propulsion
from aviary.utils.test_utils.variable_test import assert_match_varnames
from aviary.utils.functions import get_path
from aviary.validation_cases.validation_tests import (flops_validation_test,
                                                      get_flops_case_names,
                                                      get_flops_inputs,
                                                      print_case)
from aviary.variable_info.variables import Aircraft


class EngineMassTest(unittest.TestCase):

    def setUp(self):
        self.prob = om.Problem()

    @parameterized.expand(get_flops_case_names(),
                          name_func=print_case)
    def test_case(self, case_name):

        prob = self.prob

        prob.model.add_subsystem(
            "engine_mass",
            EngineMass(aviary_options=get_flops_inputs(case_name)),
            promotes_inputs=['*'],
            promotes_outputs=['*'],
        )

        prob.setup(check=False, force_alloc_complex=True)

        flops_validation_test(
            prob,
            case_name,
            input_keys=[Aircraft.Engine.SCALED_SLS_THRUST,
                        Aircraft.Engine.MASS,
                        Aircraft.Engine.ADDITIONAL_MASS,
                        Aircraft.Propulsion.TOTAL_ENGINE_MASS],
            output_keys=[Aircraft.Engine.MASS,
                         Aircraft.Engine.ADDITIONAL_MASS,
                         Aircraft.Propulsion.TOTAL_ENGINE_MASS],
            list_inputs=True,
            list_outputs=True,
            rtol=1e-10)

    def test_case_2(self):
        # arbitrary case to trigger both types of scaling equations
        prob = om.Problem()

        options = AviaryValues()

        options.set_val(Aircraft.Engine.REFERENCE_MASS, 6000, units='lbm')
        options.set_val(Aircraft.Engine.NUM_ENGINES, 2)
        options.set_val(Aircraft.Engine.SCALE_MASS, True)
        options.set_val(Aircraft.Engine.MASS_SCALER, 1.15)
        options.set_val(Aircraft.Engine.ADDITIONAL_MASS_FRACTION, 0.9)
        options.set_val(Aircraft.Engine.IGNORE_NEGATIVE_THRUST, False)
        options.set_val(Aircraft.Engine.GENERATE_FLIGHT_IDLE, True)
        options.set_val(Aircraft.Engine.GEOPOTENTIAL_ALT, False)
        options.set_val(Aircraft.Engine.FLIGHT_IDLE_THRUST_FRACTION, 0.0)
        options.set_val(Aircraft.Engine.FLIGHT_IDLE_MAX_FRACTION, 1.0)
        options.set_val(Aircraft.Engine.FLIGHT_IDLE_MIN_FRACTION, 0.08)

        options.set_val(Aircraft.Engine.DATA_FILE, get_path(
            'models/engines/turbofan_28k.deck'))
        engine = EngineDeck(options=options)
        options.set_val(Aircraft.Engine.SCALE_MASS, False)
        engine2 = EngineDeck(name='engine2', options=options)
        options.set_val(Aircraft.Engine.MASS_SCALER, 0.2)
        options.set_val(Aircraft.Engine.SCALE_MASS, True)
        engine3 = EngineDeck(name='engine3', options=options)
        preprocess_propulsion(options, [engine, engine2, engine3])

        prob.model.add_subsystem('engine_mass', EngineMass(
            aviary_options=options), promotes=['*'])
        prob.setup(force_alloc_complex=True)
        prob.set_val(Aircraft.Engine.SCALED_SLS_THRUST,
                     np.array([28000] * 3), units='lbf')

        prob.run_model()

        mass = prob.get_val(Aircraft.Engine.MASS, 'lbm')
        total_mass = prob.get_val(Aircraft.Propulsion.TOTAL_ENGINE_MASS, 'lbm')
        additional_mass = prob.get_val(Aircraft.Engine.ADDITIONAL_MASS, 'lbm')

        mass_expected = np.array([5779.16494294, 6000.0, 5814.38])
        total_mass_expected = np.array([35187.08988589])
        additional_mass_expected = np.array([5201.24844865, 5400., 5232.942])

        assert_near_equal(mass, mass_expected, tolerance=1e-10)
        assert_near_equal(total_mass, total_mass_expected, tolerance=1e-10)
        assert_near_equal(additional_mass, additional_mass_expected, tolerance=1e-10)

    def test_IO(self):
        assert_match_varnames(self.prob.model)


if __name__ == "__main__":
    unittest.main()



import numpy as np
import unittest

from numpy.testing import assert_almost_equal
from openmdao.utils.testing_utils import use_tempdirs

from aviary.interface.methods_for_level2 import AviaryProblem

from aviary.subsystems.test.test_dummy_subsystem import (
    PostOnlyBuilder, ArrayGuessSubsystemBuilder, AdditionalArrayGuessSubsystemBuilder,
    Mission, MoreMission)


@use_tempdirs
class TestSubsystemsMission(unittest.TestCase):
    def setUp(self):
        self.phase_info = {
            'pre_mission': {
                'include_takeoff': False,
                'external_subsystems': [ArrayGuessSubsystemBuilder(), AdditionalArrayGuessSubsystemBuilder()],
                'optimize_mass': True,
            },
            'cruise': {
                "subsystem_options": {"core_aerodynamics": {"method": "computed"}},
                'external_subsystems': [ArrayGuessSubsystemBuilder(), AdditionalArrayGuessSubsystemBuilder()],
                "user_options": {
                    "optimize_mach": False,
                    "optimize_altitude": False,
                    "polynomial_control_order": 1,
                    "num_segments": 2,
                    "order": 3,
                    "solve_for_distance": False,
                    "initial_mach": (0.72, "unitless"),
                    "final_mach": (0.72, "unitless"),
                    "mach_bounds": ((0.7, 0.74), "unitless"),
                    "initial_altitude": (35000.0, "ft"),
                    "final_altitude": (35000.0, "ft"),
                    "altitude_bounds": ((23000.0, 38000.0), "ft"),
                    "throttle_enforcement": "boundary_constraint",
                    "fix_initial": True,
                    "constrain_final": False,
                    "fix_duration": False,
                    "initial_bounds": ((0.0, 0.0), "min"),
                    "duration_bounds": ((10., 30.), "min"),
                },
                "initial_guesses": {"times": ([0, 30], "min")},
            },
            'post_mission': {
                'include_landing': False,
                'external_subsystems': [PostOnlyBuilder()],
            }
        }

    def test_subsystems_in_a_mission(self):
        phase_info = self.phase_info.copy()

        prob = AviaryProblem()

        prob.load_inputs("models/test_aircraft/aircraft_for_bench_GwFm.csv", phase_info)

        # Preprocess inputs
        prob.check_and_preprocess_inputs()

        prob.add_pre_mission_systems()

        prob.add_phases()

        prob.add_post_mission_systems()

        # Link phases and variables
        prob.link_phases()

        prob.add_driver("SLSQP", max_iter=0)

        prob.add_design_variables()

        prob.add_objective('fuel')

        prob.setup()

        prob.phase_info['cruise']['initial_guesses'][f'states:{Mission.Dummy.VARIABLE}'] = ([
            10., 100.], 'm')
        prob.set_initial_guesses()

        # add an assert to see if the initial guesses are correct for Mission.Dummy.VARIABLE
        assert_almost_equal(prob[f'traj.phases.cruise.states:{Mission.Dummy.VARIABLE}'], [[10.],
                                                                                          [25.97729616],
                                                                                          [48.02270384],
                                                                                          [55.],
                                                                                          [70.97729616],
                                                                                          [93.02270384],
                                                                                          [100.]])

        prob.run_aviary_problem()

        # add an assert to see if MoreMission.Dummy.TIMESERIES_VAR was correctly added to the dymos problem
        assert_almost_equal(prob[f'traj.phases.cruise.timeseries.{MoreMission.Dummy.TIMESERIES_VAR}'], np.array(
            [[0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5]]).T)

    def test_bad_initial_guess_key(self):
        phase_info = self.phase_info.copy()
        phase_info['cruise']['initial_guesses']['bad_guess_name'] = ([10., 100.], 'm')

        prob = AviaryProblem(reports=False)

        prob.load_inputs("models/test_aircraft/aircraft_for_bench_GwFm.csv", phase_info)

        # Preprocess inputs
        prob.check_and_preprocess_inputs()

        prob.add_pre_mission_systems()

        with self.assertRaises(TypeError) as context:
            prob.add_phases()
        print(str(context.exception))
        self.assertTrue(
            'EnergyPhase: cruise: unsupported initial guess: bad_guess_name' in str(context.exception))


if __name__ == "__main__":
    unittest.main()
